<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<!-- Object: vcmTransform.xsl/1.1 -->

<!-- Transform Configuration File
     ============================
     This stylesheet gets included in the vcmConfiguration.xml file and drives the transformation
     process that converts abstract elements into their corresponding primitive elements and creates
     the output file.
-->


<!-- Pretty print the output so we can read it -->
<xsl:output method="xml" indent="yes"/>


<!-- Include elements
     ================
     For each include element, use the whole content of the include element (.) as the filename for
     the document() method.  Select everything inside the VCm element and apply templates as normal.  
-->
<xsl:template match="include">
	<xsl:apply-templates select="document(.)/VCm/*"/>
</xsl:template>


<!-- Main Element
     ============
     Process the main <VCm> element and then apply all other templates.
-->
<xsl:template match="/VCm">
<xsl:text disable-output-escaping="yes">&lt;?xml-stylesheet type="text/xsl" href="vcmConfigurationXmlToG.xsl"?&gt;</xsl:text>
  <VCm
    description='{@description}'
    timeStamp='{@timeStamp}'
    >
    <xsl:apply-templates/>
  </VCm>
</xsl:template>


<!-- Copy Verbatim
     =============
     The following template copies everthing verbatim, but applies any relevant templates along the
     way.  This way, elements like <allSystems/> can be used wherever you need them.
-->
<xsl:template match="@*|*">
  <xsl:copy>
  <xsl:apply-templates select="@*|node()" />
  </xsl:copy>
</xsl:template>


<!-- Don't copy stylesheet
     =====================
     This line ensures that the xsl:stylesheet tree itself does *not* get copied into the output document 
-->
<xsl:template match="xsl:stylesheet"/>


<!-- ============================== -->
<!-- Element Specific Transforms    -->
<!-- ============================== -->


<!-- <location> Element
     ==================
     Transform a <location> element into a <logicalLocation> and a set of <physicalLocation> elements.
     Ideally we should use an absolute match here (eg /VCm/location), however this would make it
     vulnerable to skipping definitions in sloppy vcmConfiguration.xml files.  For example if
     the <sandboxes> element in /VCm/sandboxes/sandbox is not suppressed you end up with /VCm/sandboxes/location
     after pass 1.  This would then not be matched in pass 2.  So we treat any <location> element anywhere
     as if it were a /VCm/location element.
-->
<xsl:template match="location">
  <xsl:apply-templates select="./componentMask" mode="physicalLocation" />
  <logicalLocation
    logicalLocation="{@location}"
    description="{@description}"
    singleOrMultiVersion="{@singleOrMultiVersion}"
    accessCodes="{@accessCodes}"
    >
    <xsl:apply-templates select="./componentMask" mode="componentMask" />
    <xsl:apply-templates/>
  </logicalLocation>
</xsl:template>

<xsl:template match="location/componentMask" mode="physicalLocation">
  <xsl:variable name="physicalLocation">
    <xsl:choose>
      <xsl:when test="@physicalLocation"><xsl:value-of select="@physicalLocation"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="../@location"/>$<xsl:value-of select="@storageFormat"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="description">
    <xsl:choose>
      <xsl:when test="@description"><xsl:value-of select="@description"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="../@description"/> for <xsl:value-of select="@componentMask"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="multiVersion">
    <xsl:choose>
      <xsl:when test="@multiVersion"><xsl:value-of select="@multiVersion"/></xsl:when>
      <xsl:when test="(../@singleOrMultiVersion)='Single'">No</xsl:when>
      <xsl:when test="(../@singleOrMultiVersion)='Multi'">Yes</xsl:when>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="locationType">
    <xsl:choose>
      <xsl:when test="@locationType"><xsl:value-of select="@locationType"/></xsl:when>
      <xsl:otherwise>Controlled</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="dependencyCheck">
    <xsl:choose>
      <xsl:when test="@dependencyCheck"><xsl:value-of select="@dependencyCheck"/></xsl:when>
      <xsl:otherwise>Internal</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <physicalLocation 
    physicalLocation="{$physicalLocation}"
    description="{$description}"
    storageFormat="{@storageFormat}"
    multiVersion="{$multiVersion}"
    physicalAddress="{@physicalAddress}"
    locationType="{$locationType}"
    dependencyCheck="{$dependencyCheck}"
    />
</xsl:template>

<xsl:template match="location/componentMask" mode="componentMask">
  <xsl:variable name="physicalLocation">
    <xsl:choose>
      <xsl:when test="@physicalLocation"><xsl:value-of select="@physicalLocation"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="../@location"/>$<xsl:value-of select="@storageFormat"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <componentMask
    maskSequence="{@maskSequence}"
    componentMask="{@componentMask}"
    physicalLocation="{$physicalLocation}"
    />
</xsl:template>

<!-- When called with no mode then translate to nothing -->
<xsl:template match="location/componentMask" />

<xsl:template match="location/system">
  <system><xsl:value-of select='.'/></system>
</xsl:template>

<!-- This rule overrides the rule for match="location", otherwise locations listed within a locationClass
     are taken as being locations which isn't what we want at all! -->
<xsl:template match="locationClass/location">
  <xsl:copy>
  <xsl:apply-templates select="@*|node()" />
  </xsl:copy>
</xsl:template>


<!-- <allSystems/> Element
     =====================
     Implementation of <allSystems/> element.  Whereever this element occurs, it will 
     be replaced by a list of all systems that are defined using the <system> element.
-->
<xsl:template match="allSystems">
	<xsl:for-each select="/VCm/system">
		<system><xsl:value-of select="@system"/></system>
	</xsl:for-each>
</xsl:template>



</xsl:stylesheet>
